<?php

/**
 * helper class for doing stuff
 * 
 * @category   Plugin
 * @package    ekomi_integration
 * @author     Sandor Barics <sbarics@ekomi.de>
 * @copyright  2015-2016 
 */

/**
 * Class ekomiIntegrationHelper
 */
class ekomiIntegrationHelper {

    /**
     * @var null|ekomiIntegrationHelper
     */
    private static $_instance = null;

    /**
     * @var null|bool
     */
    private static $_isModern = null;

    /**
     * @var null|NiceDB
     */
    private $db = null;

    /**
     * @var null|Plugin
     */
    private $plugin = null;

    /**
     * order Object
     * @var mix 
     */
    var $order = null;

    /**
     * constructor
     *
     * @param Plugin $oPlugin
     */
    public function __construct(Plugin $oPlugin) {
        $this->plugin = $oPlugin;
        //get database instance - do not do this, use Shop::DB()/$GLOBALS['DB'] instead
        if (self::isModern()) {
            $this->db = Shop::DB();
        } else {
            $this->db = $GLOBALS['DB'];
        }
    }

    /**
     * singleton getter
     *
     * @param Plugin $oPlugin
     * @return ekomiIntegrationHelper
     */
    public static function getInstance(Plugin $oPlugin) {
        return (self::$_instance === null) ? new self($oPlugin) : self::$_instance;
    }

    /**
     * Gets the configuration settings
     * 
     * @param type $kPlugin          The Plugin id
     * @param type $kPluginAdminMenu The Plugin admin Id
     * 
     * @return array configuration settings array
     */
    public function fetchConfigSettings($kPlugin, $kPluginAdminMenu) {
        $oPluginEinstellungConf_arr = $this->db->query(
                "SELECT *
                        FROM tplugineinstellungenconf
                        WHERE kPluginAdminMenu != 0
                            AND kPlugin = " . $kPlugin . "
                            AND cConf = 'Y'
                            AND kPluginAdminMenu = " . $kPluginAdminMenu, 2
        );

        return $oPluginEinstellungConf_arr;
    }

    /**
     * Deletes the config setting
     * 
     * @param type $kPlugin   The plugin id
     * @param type $cWertName The config setting key
     * 
     * @return boolean True if success otherwise false
     */
    public function deleteConfig($kPlugin, $cWertName) {
        return $this->db->delete('tplugineinstellungen', array('kPlugin', 'cName'), array($kPlugin, $cWertName));
    }

    /**
     * Saves the config setting
     * 
     * @param type $kPlugin   The plugin id
     * @param type $cWertName The config setting key
     * @param type $cWert     The config setting value
     * 
     * @return boolean True if success otherwise false
     */
    public function saveConfig($kPlugin, $cWertName, $cWert) {
        $oPluginEinstellung = new stdClass();
        $oPluginEinstellung->kPlugin = $kPlugin;
        $oPluginEinstellung->cName = $cWertName;
        $oPluginEinstellung->cWert = $cWert;

        return $this->db->insert('tplugineinstellungen', $oPluginEinstellung);
    }

    /**
     * Converts the array to associative array
     * 
     * @param type $objectArray  The array
     * 
     * @return array The associative array
     */
    public function toAssociativeArray($objectArray) {

        $associativeArr = array();
        if ($objectArray) {
            foreach ($objectArray as $key => $value) {
                $associativeArr[$value->cName] = $value->cWert;
            }
        }

        return $associativeArr;
    }

    /**
     * check if there is a current shop version installed
     *
     * @return bool
     */
    public static function isModern() {
        if (self::$_isModern === null) {
            //cache the actual value as class variable
            self::$_isModern = version_compare(JTL_VERSION, 400, '>=') && class_exists('Shop');
        }

        return self::$_isModern;
    }

    /**
     * Initialises the order object
     * 
     * @param int $orderId The order id
     */
    function initOrder($orderId) {
        $this->order = $this->_fetchOrder($orderId);
    }

    /**
     * Gets the order status
     * 
     * @return int order status id Otherwise null
     */
    function getOrderStatus() {
        if (isset($this->order->cStatus)) {
            return $this->order->cStatus;
        } else {
            NULL;
        }
    }

    /**
     * Sets the order status
     * 
     * @param int $orderStatus The order status
     * 
     */
    function setOrderStatus($orderStatus) {
        if (isset($this->order->cStatus)) {
            $this->order->cStatus = $orderStatus;
        }
    }

    /**
     * Gets the order data
     * 
     * @param int    $shopId       The shop Id
     * @param string $shopPassword The shop password
     * 
     * @return string The parameters
     */
    function getOrderData($shopId, $shopPassword) {
        $postVars = '';

        $scheduleTime = date('d-m-Y H:i:s', strtotime($this->getOrdertimestamp()));

        $senderName = $this->getStoreName();

        $customer = $this->_getCustomerInfo();

        $custPhoneNum = $this->_getCustomerPhoneNumber($customer);

        if ($customer) {
            $apiMode = $this->getRecipientType($custPhoneNum);

            if ($apiMode == 'sms' && strlen($senderName) > 11)
                $senderName = substr($senderName, 0, 11);

            $fields = array(
                'shop_id' => $shopId,
                'password' => $shopPassword,
                'recipient_type' => $apiMode,
                'salutation' => '',
                'first_name' => $customer->cVorname,
                'last_name' => trim(entschluesselXTEA($customer->cNachname)),
                'email' => $customer->cMail,
                'transaction_id' => $this->order->cBestellNr,
                'transaction_time' => $scheduleTime,
                'telephone' => $custPhoneNum,
                'sender_name' => $senderName,
                'sender_email' => $this->getStoreEmail()
            );

            $fields['client_id'] = $this->order->kKunde;
            $fields['screen_name'] = $customer->cVorname . ' ' . trim(entschluesselXTEA($customer->cNachname));

            if ($this->plugin->oPluginEinstellungAssoc_arr['ekomi_integration_product_base'] == 'Y') {
                $fields['has_products'] = 1;
                $productsData = $this->getProductsData();
                $fields['products_info'] = json_encode($productsData['product_info']);
                $fields['products_other'] = json_encode($productsData['other']);
            }

            $counter = 1;
            foreach ($fields as $key => $value) {
                if ($counter > 1){
                    $postVars .= "&";
                }
                $postVars .= $key . "=" . $value;
                $counter++;
            }
        }

        return $postVars;
    }

    /**
     * Gets the products data.
     * 
     * @return array The products array
     * 
     * @access protected
     */
    protected function getProductsData()
    {
        $orderProducts = $this->_fetchOrderProducts($this->order->kWarenkorb);
        $products = array();
        $basePath = Shop::getURL() . '/';
        $configuredIdentifier = $this->plugin->oPluginEinstellungAssoc_arr['ekomi_integration_product_identifier'];
        foreach ($orderProducts as $product) {
            $kArtikel = $product->kArtikel;
            $productIdentifier = $this->getProductIdentifier($product, $configuredIdentifier);
            $products['product_info'][$productIdentifier] = $product->cSeo;
            $productOther = array();
            $imageUrl = $basePath . PFAD_MEDIA_IMAGE . "/product/{$kArtikel}/md/{$product->cSeo}.jpg";
            $productOther['image_url'] = utf8_decode($imageUrl);
            $productOther['brand_name'] = '';
            $productOther['product_ids'] = array('gbase' => utf8_decode($productIdentifier));
            $canonicalUrl = $basePath . $product->cSeo;
            $productOther['links'] = array(
                array('rel' => 'canonical', 'type' => 'text/html',
                    'href' => utf8_decode($canonicalUrl))
            );
            $products['other'][$productIdentifier]['product_other'] = $productOther;
        }

        return $products;
    }

    /**
     * Gets the recipient type
     * 
     * @param string $telephone The phone number
     * 
     * @return string Recipient type
     * 
     * @access protected
     */
    protected function getRecipientType($telephone) {

        $reviewMod = $this->plugin->oPluginEinstellungAssoc_arr['ekomi_integration_review_mod'];
        $apiMode = 'email';
        switch ($reviewMod) {
            case 'sms':
                $apiMode = 'sms';
                break;
            case 'email':
                $apiMode = 'email';
                break;
            case 'fallback':
                if ($this->validateE164($telephone))
                    $apiMode = 'sms';
                else
                    $apiMode = 'email';
                break;
        }

        return $apiMode;
    }

    /**
     * Gets the order timestamp
     * 
     * @return string The timestamp
     * 
     * @access protected
     */
    protected function getOrdertimestamp() {
        $order_date = $this->order->dErstellt;
        $order_date_array = explode(' ', $order_date);

        return $order_date_array[0];
    }

    /**
     * Gets the Shop Name
     * 
     * @return string The shop name
     */
    function getStoreName() {
        return ''; //STORE_NAME;
    }

    /**
     * Gets the shop email id
     * 
     * @return string The email id
     */
    function getStoreEmail() {
        return ''; //STORE_OWNER_EMAIL_ADDRESS;
    }

    /**
     * 
     * @param type $customer The customer object
     * 
     * @return string The customer phone number
     */
    function _getCustomerPhoneNumber($customer) {
        if (isset($customer->cMobil) && !empty($customer->cMobil)) {
            return $customer->cMobil;
        } else if (isset($customer->cTel) && !empty($customer->cTel)) {
            return $customer->cTel;
        }

        return '';
    }

    /**
     * Gets the customer object
     * 
     * @return array The customer object otherwise null
     * 
     * 
     * @access protected
     */
    protected function _getCustomerInfo() {
        $kRechnungsadresse = $this->order->kRechnungsadresse;

        $data = $this->db->query(
                "SELECT *
                        FROM trechnungsadresse
                        WHERE kRechnungsadresse = " . $kRechnungsadresse, 2
        );
        if ($data && isset($data[0])) {
            return $data[0];
        }
        return null;
    }

    /**
     * Validates E164 numbers
     * 
     * @param $phone The phone number
     *
     * @return bool Yes if matches False otherwise
     * 
     * @access protected
     */
    protected function validateE164($phone) {
        $pattern = '/^\+?[1-9]\d{1,14}$/';

        preg_match($pattern, $phone, $matches);

        if (!empty($matches)) {
            return true;
        }

        return false;
    }

    /**
     * Fetches the order data
     * 
     * @param int $order_id The order id
     * 
     * @return array The order data
     * 
     * @access protected
     */
    protected function _fetchOrder($order_id) {

        $data = $this->db->query(
                "SELECT *
                        FROM tbestellung
                        WHERE cBestellNr = '" . $order_id. "'", 2
        );

        if ($data && isset($data[0])) {
            return $data[0];
        }
        
        return null;
    }

    /**
     * Fetches the ordered products
     * 
     * @param type $order_id The order id
     * 
     * @return array  The products data
     * 
     * @access protected
     */
    protected function _fetchOrderProducts($cartID) {
        $query = "SELECT twarenkorbpos.kArtikel, tartikel.cArtNr, tartikel.cName, tartikel.cSeo 
                    FROM twarenkorbpos
                    INNER JOIN tartikel ON twarenkorbpos.kArtikel = tartikel.kArtikel
                   WHERE kWarenkorb={$cartID}";

        $data = $this->db->query($query, 2);
        if ($data && is_array($data)) {
            return $data;
        }
        return null;
    }

    /**
     * Gets product identifiers.
     */
    public function getProductIdentifierList()
    {
        $productIdentifiers = array(
            'kArtikel' => 'ID',
            'cArtNr' => 'Number',
            'cISBN' => 'ISBN',
            'cUPC' => 'UPC'
        );

        return $productIdentifiers;
    }

    /**
     * Gets product identifier.
     *
     * @param array  $product
     * @param string $configuredIdentifier
     *
     * @return int|string
     */
    public function getProductIdentifier($product, $configuredIdentifier)
    {
        if (!empty($configuredIdentifier)) {
            return $product->$configuredIdentifier;
        }

        return $product->kArtikel;
    }

}
